-------------------------------------------------------------------
-- Date			: 13-Aug-2012
-- Database		: DB2 9.7
-- Description	: ISAMESSO Enterprise ID / Domain Update Script
-- IMS Version	: 8.1.x / 8.2.x
--
-- Assumption :-
--   - NEW enterprise directory is configured in ISAMESSO
--
-- Notes :-
--   - Changes on user domain will requires manual changes on the new access profile that rely on enterprise directory
--   - Changes on enterprise id / user domain will trigger password change flow at AA's side
--	 - Change of UPN will affects :
--		 -- Smart Card (00D)
--		 -- AA/AAWWP login with UPN
--
-- Tables affected : 
--  - IMSIdentityAttribute 
--  - IMSIdentityUniqueAttribute
--	- IMSTemplatedSyncRepo
--	- IMSSoci 
--	- IMSSociAttribute
--	- IMSSociUniqueAttribute
--  - IMSCertificate 
--
-- Changes Log :
-- v1.0.1 - Replace NVARCHAR to VARCHAR for better compatiblity on different DB2 version
--
-- call sp_base_updEntIdDomainUpnGuid ('SIMULATE', 'hlong-test02', 'qa.encentuate.com', 'hlong-test02', 'song.ims82.com', '', '')
-- call sp_base_updEntIdDomainUpnGuid ('ABOUT', '<CUR-UID>', '<CUR-DOMAIN>', '<NEW-UID>', '<CUR-DOMAIN>', '<NEW-UPN>', '<NEW-ADGUID>')
-- call sp_base_updEntIdDomainUpnGuid ('HELP', '<CUR-UID>', '<CUR-DOMAIN>', '<NEW-UID>', '<CUR-DOMAIN>', '<NEW-UPN>', '<NEW-ADGUID>')
-- call sp_base_updEntIdDomainUpnGuid ('INVALID', '<CUR-UID>', '<CUR-DOMAIN>', '<NEW-UID>', '<CUR-DOMAIN>', '<NEW-UPN>', '<NEW-ADGUID>')
-- drop procedure sp_base_updEntIdDomainUpnGuid
--
CREATE PROCEDURE sp_base_updEntIdDomainUpnGuid
(
 IN @IN_operationCmd VARCHAR(8)
 
,IN @IN_curUserEntId VARCHAR(128)
,IN @IN_curDomain VARCHAR(128)

,IN @IN_newUserEntId VARCHAR(128)
,IN @IN_newDomain VARCHAR(128)

,IN @IN_newUPN VARCHAR(128)
,IN @IN_newAdGUID VARCHAR(128) --AD ONLY

,OUT @var_showSyntax CHAR(1) 
,IN @IN_exec_id VARCHAR(128) 
,IN @IN_scriptName VARCHAR(64)
,IN @IN_scriptDesc VARCHAR(64)
,IN @IN_scriptVer VARCHAR(32)
)
DYNAMIC RESULT SETS 1
MODIFIES SQL DATA
LANGUAGE SQL 
P1 : BEGIN
		DECLARE @var_base_script_info VARCHAR(128) DEFAULT 'sp_base_updEntIdDomainUpnGuid 1.0.1';
		DECLARE @var_script_name VARCHAR(128);
		DECLARE @var_script_desc VARCHAR(128);
		DECLARE @var_script_ver VARCHAR(32);
	
		DECLARE @var_exec_id VARCHAR (128);
		DECLARE @var_msg_id INTEGER DEFAULT 100;
		DECLARE TMPTBL_ALREADY_EXIST CONDITION FOR SQLSTATE '42710';
		DECLARE CONTINUE HANDLER FOR TMPTBL_ALREADY_EXIST
		BEGIN 
			-- DO NOTHING
		END;
		DECLARE GLOBAL TEMPORARY TABLE SESSION.tmpDbmsOutput
		(
			EXECID VARCHAR (32) NOT NULL,
			MSGID INTEGER NOT NULL DEFAULT 0,
			MSG VARCHAR (1024) NOT NULL
		) NOT LOGGED ON COMMIT PRESERVE ROWS;	
			
		IF(@IN_exec_id = NULL OR LENGTH(TRIM(@IN_exec_id))=0) THEN
			SELECT CHAR(CURRENT TIMESTAMP) INTO @var_exec_id FROM SYSIBM.SYSDUMMY1;
		ELSE
			SET @var_exec_id = @IN_exec_id;
		END IF;
		
		SET @var_script_name 	= @IN_scriptName;
		SET @var_script_desc 	= @IN_scriptDesc;
		SET @var_script_ver 	= @IN_scriptVer;
		
P2 : BEGIN	

	DECLARE @var_cur_user_entid VARCHAR(128);
	DECLARE @var_new_user_entid VARCHAR(128);
	
	DECLARE @var_cur_upn VARCHAR(256);
	DECLARE @var_new_upn VARCHAR(256);

	DECLARE @var_cur_domain VARCHAR(128);
	DECLARE @var_new_domain VARCHAR(128);
		
	DECLARE @var_cur_ims_user_ent_login VARCHAR(256);
	DECLARE @var_new_ims_user_ent_login VARCHAR(256);

	DECLARE @var_cur_ims_authsrv_id VARCHAR(128);	
	DECLARE @var_new_ims_authsrv_id VARCHAR(128);	

	DECLARE @var_cur_ad_userguid VARCHAR(128);	
	DECLARE @var_new_ad_userguid VARCHAR(128);	

	DECLARE @var_imsid VARCHAR(256);
	DECLARE @var_cur_ims_entdir_id VARCHAR(128);
	DECLARE @var_new_ims_entdir_id VARCHAR(128);	
	DECLARE @var_is_revoked_cwallet CHAR(1) DEFAULT 'N';
	--- 
	DECLARE @var_info_db_ver VARCHAR(128);
	DECLARE @var_valid_schema CHAR(1) DEFAULT 'N';
	DECLARE @var_tmpstr VARCHAR(512);
	DECLARE @var_tmpstr2 VARCHAR(512);
	---
	DECLARE @var_iTmpNum INTEGER;
	DECLARE @val_message varchar(1024);
	DECLARE @exception_msg varchar(128) DEFAULT NULL;

	DECLARE @var_tmpstr_notfound VARCHAR(16) DEFAULT '<NOT FOUND>';
	DECLARE @var_tmpstr_nochanges VARCHAR(16) DEFAULT '(remain)';	
	
	DECLARE @cursor_dbms_output CURSOR WITH RETURN TO CLIENT FOR
	SELECT MSG FROM SESSION.tmpDbmsOutput WHERE EXECID = @var_exec_id ORDER BY MSGID;

	-- Handle SQLException
	DECLARE EXIT HANDLER FOR SQLEXCEPTION
	BEGIN
		GET DIAGNOSTICS EXCEPTION 1 @exception_msg = MESSAGE_TEXT;
		
		IF @val_message IS NULL THEN
			SET @val_message = @exception_msg;
		END IF;
		
		INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id,@val_message);
		
		OPEN @cursor_dbms_output;
		
	END;
		
	--==============================================================
	---
	SET @var_cur_user_entid = @IN_curUserEntId;
	SET @var_cur_domain = @IN_curDomain;
	
	SET @var_new_user_entid = @IN_newUserEntId;
	SET @var_new_domain = @IN_newDomain;	

	SET @var_new_upn = @IN_newUPN;
	
	SET @var_new_ad_userguid = @IN_newAdGUID;
	----
	SET @var_is_revoked_cwallet	= 1; -- always revoke cached wallet
	----
	
	--==============================================================
	-- verify IMS schema
	SELECT COUNT(TABNAME) INTO @var_iTmpNum FROM SYSCAT.TABLES 
	WHERE UPPER(TABNAME) IN ('IMSIDENTITYATTRIBUTE', 'IMSIDENTITYUNIQUEATTRIBUTE', 'IMSTEMPLATEDSYNCREPO',
							  'IMSSOCI', 'IMSSOCIATTRIBUTE', 'IMSSOCIUNIQUEATTRIBUTE', 
							  'IMSCERTIFICATE')
	AND TABSCHEMA = CURRENT SCHEMA;

	IF (@var_iTmpNum = 7) THEN
		SET @var_valid_schema = 'Y';
	ELSE		
		SET @var_valid_schema = 'N';
	END IF;
	
	IF (@var_valid_schema = 'Y') THEN
	
			-- Get current domain authSrvId 
			SELECT authSrvID INTO @var_cur_ims_authsrv_id FROM IMSEntDirAppMap 
				WHERE dnsDomain = @var_cur_domain;
				
			-- Get new domain authSrvId 
			SELECT authSrvID INTO @var_new_ims_authsrv_id FROM IMSEntDirAppMap 
				WHERE dnsDomain = @var_new_domain;
	
			-- Get current domain entDirID 
			SELECT entDirID INTO @var_cur_ims_entdir_id FROM IMSEntDirAppMap 
				WHERE dnsDomain = @var_cur_domain;

			-- Get new domain entDirID 
			SELECT entDirID INTO @var_new_ims_entdir_id FROM IMSEntDirAppMap 
				WHERE dnsDomain = @var_new_domain;
				
			-- Get current UPN 
			SELECT attrValue INTO @var_cur_upn FROM IMSIdentityUniqueAttribute
				WHERE imsID = @var_imsid AND attrName = 'EnterpriseUpn';
	
			IF (@var_new_ad_userguid IS NOT NULL) THEN
				-- Get current AD-ONLY userGUID
				SELECT attrValue INTO @var_cur_ad_userguid FROM IMSIdentityUniqueAttribute
					WHERE imsID = @var_imsid AND attrName = 'userGUID';
				
				-- Format userGUID String into IMS format 
				-- {90D0A2EB-5047-492D-BBE9-6D33C6706D30} -> 90d0a2eb5047492dbbe96d33c6706d30
				SET @var_new_ad_userguid = REPLACE(REPLACE(LOWER(REPLACE(@var_new_ad_userguid,'-','')),'{',''),'}','');
			END IF;
			
			-- Retrive imsId of target user		
			SET @var_cur_ims_user_ent_login = @var_cur_domain || '\' || @var_cur_user_entid;
			SET @var_new_ims_user_ent_login = @var_new_domain || '\' || @var_new_user_entid;
	
			--*1. Try 'domain.com\userid' format
			SELECT imsID INTO @var_imsid FROM IMSIdentityUniqueAttribute 
				WHERE attrName='Enterprise Login' AND attrValue = @var_cur_ims_user_ent_login;

			IF (@var_imsid IS NULL) THEN
				--*2. Try 'edir_xxx\userid' format
				SELECT imsID INTO @var_imsid FROM IMSIdentityUniqueAttribute 
					WHERE attrName='Enterprise Login' AND attrValue = @var_cur_ims_entdir_id || '\' || @var_cur_user_entid;
					
				IF (@var_imsid IS NULL) THEN
					SET @var_cur_ims_user_ent_login = @var_cur_ims_entdir_id || '\' || @var_cur_user_entid;
					SET @var_new_ims_user_ent_login = @var_new_ims_entdir_id || '\' || @var_new_user_entid;		
				END IF;
			END IF;			
			
			-- Assign current UPN as new UPN if it's NULL (remain unchange)
			IF @var_new_upn IS NULL THEN
				SET @var_new_upn = @var_cur_upn;
			END IF;
			--==============================================================
			IF @IN_operationCmd = 'ABOUT' THEN

				INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id+1, @var_script_desc || ' Script');
				INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id+5,'  - Script Name		 : ' || @var_script_name);
				INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id+6,'  - Script Version	 : ' || @var_script_ver);
				INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id+6,'  - Base Script Info : ' || @var_base_script_info);
				INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id+2,'  - Database Name	 : ' || CURRENT SERVER);
				INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id+3,'  - Database Version : ' || (SELECT service_level FROM TABLE (sysproc.env_get_inst_info())));
				INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id+4,'  - Schema Name		 : ' || CURRENT SCHEMA);
			
				--SELECT INTEGER('A') INTO @var_iTmpNum FROM SYSIBM.SYSDUMMY1;							

			ELSEIF @IN_operationCmd = 'HELP' THEN
				SET @var_showSyntax = 'Y';			
				
			ELSEIF @IN_operationCmd IN ('SIMULATE','UPDATE') THEN
			
				-- Validate input with DB 
				IF (@var_imsid IS NULL OR LENGTH(@var_imsid)=0)
					OR(@var_new_domain IS NULL OR LENGTH(@var_new_domain)=0)
					OR(@var_new_user_entid IS NULL OR LENGTH(@var_new_user_entid)=0)
					OR(@var_new_domain=@var_cur_domain AND @var_new_user_entid=@var_cur_user_entid)
					OR(@var_cur_ims_authsrv_id IS NULL OR LENGTH(@var_cur_ims_authsrv_id)=0)
					OR(@var_new_ims_entdir_id IS NULL) THEN
					
					IF (@var_imsid IS NULL) THEN
						SET @var_msg_id = @var_msg_id + 1;
						SET @val_message = '[ERROR] Invalid current enterprise id or domain ! ''' || @var_cur_ims_user_ent_login || ''' is NOT found in ISAMESSO.';
						INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id,@val_message);
					END IF;
					
					IF (@var_cur_ims_authsrv_id IS NULL) THEN
						SET @var_msg_id = @var_msg_id + 1;
						SET @val_message = '[ERROR] Invalid current user domain ! ''' || @var_cur_domain || ''' is NOT found in ISAMESSO.';
						INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id,@val_message);
					END IF;
			
					IF (@var_new_domain IS NULL OR LENGTH(@var_new_domain)=0) THEN
						SET @var_msg_id = @var_msg_id + 1;
						SET @val_message = '[ERROR] New user domain cannot be empty ! ';
						INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id,@val_message);
						
					ELSEIF (@var_new_ims_entdir_id IS NULL OR LENGTH(@var_new_ims_entdir_id)=0) THEN
						SET @var_msg_id = @var_msg_id + 1;
						SET @val_message = '[ERROR] Invalid new user domain ! New domain ''' || @var_new_domain || ''' is NOT configured in ISAMESSO.';
						INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id,@val_message);
					END IF;
							
					IF (@var_new_domain=@var_cur_domain AND @var_new_user_entid=@var_cur_user_entid) THEN
						SET @var_msg_id = @var_msg_id + 1;
						SET @val_message = '[ERROR] New & current user enterprise ID / user domain must be different. ';
						INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id,@val_message);
					END IF;
					
					--SELECT INTEGER('A') INTO @var_iTmpNum FROM SYSIBM.SYSDUMMY1;
					
				ELSE -- OK
				
					IF @IN_operationCmd = 'UPDATE' THEN
						
						-- Check if Smart Card in used
						SELECT CHAR(COUNT(*)) INTO @var_tmpstr FROM IMSSoci WHERE imsID = @var_imsid AND sociID LIKE '00D%';
						
						IF(@var_tmpstr <> '0') THEN
							SET @var_msg_id = @var_msg_id + 1;
							SET @val_message = '[ERROR] Smart card record found ! This script is NOT certified for smart card deployment ! ';
							INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id,@val_message);
						END IF;
	
						BEGIN ATOMIC
							--[ IMSIdentityAttribute ]---------------------------------------------------
							
							IF (@var_new_user_entid <> @var_cur_user_entid) THEN
							
								-- Update EnterpriseUsername
								UPDATE IMSIdentityAttribute 
								SET attrValue = @var_new_user_entid
								WHERE imsID = @var_imsid
									AND attrName = 'EnterpriseUsername'
									AND attrValue = @var_cur_user_entid;
								--
								GET DIAGNOSTICS @var_iTmpNum = ROW_COUNT;							
								SET @var_msg_id = @var_msg_id + 1;
								SET @val_message = '   Update IMSIdentityAttribute.EnterpriseUsername to ' || @var_new_user_entid || '. - '||CHAR(@var_iTmpNum);
								INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id,@val_message);
							END IF;
							
							IF(@var_new_ims_entdir_id <> @var_cur_ims_entdir_id) THEN
	
								-- Update userEnterpriseDir
								UPDATE IMSIdentityAttribute 
								SET attrValue = @var_new_ims_entdir_id
								WHERE imsID = @var_imsid
									AND attrName = 'userEnterpriseDir'
									AND attrValue = @var_cur_ims_entdir_id;
								--
								GET DIAGNOSTICS @var_iTmpNum = ROW_COUNT;							
								SET @var_msg_id = @var_msg_id + 1;
								SET @val_message = '   Update IMSIdentityAttribute.userEnterpriseDir to '|| @var_new_ims_entdir_id || '. - '|| CHAR(@var_iTmpNum);
								INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id,@val_message);
							END IF;
							
							IF(@var_new_domain <> @var_cur_domain) THEN
								-- Update userDomain
								UPDATE IMSIdentityAttribute 
								SET attrValue = @var_new_domain
								WHERE imsID = @var_imsid
									AND attrName = 'userDomain'
									AND attrValue = @var_cur_domain;
								--
								GET DIAGNOSTICS @var_iTmpNum = ROW_COUNT;							
								SET @var_msg_id = @var_msg_id + 1;
								SET @val_message = '   Update IMSIdentityAttribute.userDomain to '|| @var_new_domain || '. - '|| CHAR(@var_iTmpNum);
								INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id,@val_message);
							END IF;							
								
							--[ IMSIdentityUniqueAttribute ]---------------------------------------------------
							IF(@var_new_ims_user_ent_login<>@var_cur_ims_user_ent_login)THEN
								-- Update Enterprise Login 
								UPDATE IMSIdentityUniqueAttribute 
								SET attrValue = @var_new_ims_user_ent_login
									,ucAttrValue = UPPER(@var_new_ims_user_ent_login) 
								WHERE imsID = @var_imsid
									AND attrName = 'Enterprise Login'
									AND attrValue = @var_cur_ims_user_ent_login;
									--
								GET DIAGNOSTICS @var_iTmpNum = ROW_COUNT;							
								SET @var_msg_id = @var_msg_id + 1;
								SET @val_message = '   Update IMSIdentityUniqueAttribute.Enterprise Login to ' || @var_new_ims_user_ent_login || '. - '|| CHAR(@var_iTmpNum);
								INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id,@val_message);
							END IF;
								
							IF (@var_new_upn <> @var_cur_upn)THEN
								-- Update EnterpriseUpn
								UPDATE IMSIdentityUniqueAttribute 
								SET attrValue = @var_new_upn
									,ucAttrValue = UPPER(@var_new_upn) 		
								WHERE imsID = @var_imsid
									AND attrName = 'EnterpriseUpn'
									AND attrValue = @var_cur_upn;
								--
								GET DIAGNOSTICS @var_iTmpNum = ROW_COUNT;							
								SET @var_msg_id = @var_msg_id + 1;
								SET @val_message = '   Update IMSIdentityUniqueAttribute.EnterpriseUpn to '||@var_new_upn||'. - '|| CHAR(@var_iTmpNum);
								INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id,@val_message);
							END IF;			
								
							IF (@var_new_ad_userguid IS NOT NULL) AND (@var_new_ad_userguid <> @var_cur_ad_userguid) THEN
								-- Update userGUID
								UPDATE IMSIdentityUniqueAttribute 
								SET attrValue = @var_new_ad_userguid
									,ucAttrValue = UPPER(@var_new_ad_userguid) 		
								WHERE imsID = @var_imsid
									AND attrName = 'userGUID'
									AND attrValue = @var_cur_ad_userguid;								
								--
								GET DIAGNOSTICS @var_iTmpNum = ROW_COUNT;							
								SET @var_msg_id = @var_msg_id + 1;
								SET @val_message = '   Update IMSIdentityUniqueAttribute.userGUID to '||@var_new_ad_userguid||'. - '|| CHAR(@var_iTmpNum);
								INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id,@val_message);
							END IF;
	
							--[ IMSTemplatedSyncRepo ]---------------------------------------------------
						
							IF (@var_new_ims_authsrv_id <> @var_cur_ims_authsrv_id)
								OR (@var_new_user_entid <> @var_cur_user_entid) THEN
							
								-- Mark AA Sync data as 'isDeleted'
								UPDATE IMSTemplatedSyncRepo
								SET updateTimeStamp = CURRENT_TIMESTAMP
									,isDeleted = 1
								WHERE attrName = 'account_data' 
									AND entityTypeID = 'scp_user'
									AND entityID = @var_imsid 
									AND keyValue LIKE '%<auth_id>'||@var_cur_ims_authsrv_id||'</auth_id><aditi_ciuser>'||@var_cur_user_entid||'</aditi_ciuser>%';
								--
								GET DIAGNOSTICS @var_iTmpNum = ROW_COUNT;							
								SET @var_msg_id = @var_msg_id + 1;
								SET @val_message = '   Mark IMSTemplatedSyncRepo.account_data as deleted. - '|| CHAR(@var_iTmpNum);
								INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id,@val_message);
								--
								
								UPDATE IMSTemplatedSyncRepo
								SET updateTimeStamp = CURRENT_TIMESTAMP
									,isDeleted = 1
								WHERE attrName = 'account_data_info' 
									AND entityTypeID = 'scp_user'
									AND entityID = @var_imsid 
									AND keyValue LIKE '%<auth_id>'||@var_cur_ims_authsrv_id||'</auth_id><aditi_ciuser>'||@var_cur_user_entid||'</aditi_ciuser>%';
								--
								GET DIAGNOSTICS @var_iTmpNum = ROW_COUNT;						
								SET @var_msg_id = @var_msg_id + 1;
								SET @val_message = '   Mark IMSTemplatedSyncRepo.account_data_info as deleted. - '|| CHAR(@var_iTmpNum);
								INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id,@val_message);
							END IF;
			
							IF (@var_new_ims_authsrv_id <> @var_cur_ims_authsrv_id) THEN
														
								-- Mark AA Sync data as 'isDeleted'
								UPDATE IMSTemplatedSyncRepo
								SET updateTimeStamp = CURRENT_TIMESTAMP
									,isDeleted = 1
								WHERE attrName = 'policy_object'
									AND entityTypeID = 'scp_user'
									AND entityID = @var_imsid 
									AND keyValue LIKE '%<eiid_auth>'||@var_cur_ims_authsrv_id||'</eiid_auth>%';	
								--
								GET DIAGNOSTICS @var_iTmpNum = ROW_COUNT;						
								SET @var_msg_id = @var_msg_id + 1;
								SET @val_message = '   Mark IMSTemplatedSyncRepo.policy_object as deleted. - '|| CHAR(@var_iTmpNum);
								INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id,@val_message);
							END IF;
			
							--[ Cached Wallet ]---------------------------------------------------
							
							IF @var_is_revoked_cwallet = 1 THEN 
								-- Delete cert 
								DELETE FROM IMSCertificate WHERE sociID IN (SELECT S.sociID FROM IMSSoci S WHERE S.sociID LIKE '200%' AND S.imsID = @var_imsid);
								-- Delete Soci Attributes 
								DELETE FROM IMSSociAttribute WHERE sociID IN (SELECT S.sociID FROM IMSSoci S WHERE S.sociID LIKE '200%' AND S.imsID = @var_imsid);
								-- Delete Soci Unique Attributes
								DELETE FROM IMSSociUniqueAttribute WHERE sociID IN (SELECT S.sociID FROM IMSSoci S WHERE S.sociID LIKE '200%' AND S.imsID = @var_imsid);
								-- Delete cached wallet 
								DELETE FROM IMSSoci WHERE sociID LIKE '200%' AND imsID = @var_imsid;
								--
								GET DIAGNOSTICS @var_iTmpNum = ROW_COUNT;						
								SET @var_msg_id = @var_msg_id + 1;
								SET @val_message = '   Clean up dangling cached wallets. - '|| CHAR(@var_iTmpNum);
								INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id,@val_message);
							END IF;
							
						END;
						COMMIT;
						SET @var_msg_id = @var_msg_id + 1;
						INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id,'Completed.');
						
					ELSEIF @IN_operationCmd = 'SIMULATE' THEN
					
						SET @var_msg_id = @var_msg_id + 1;
						SET @val_message = '  - ISAMESSO ID : ''' || COALESCE(@var_imsid,@var_tmpstr_notfound) || '''';
						INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id,@val_message);
								
						IF @var_imsid IS NOT NULL THEN
		
							SET @var_msg_id = @var_msg_id + 1;
							INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id,'  - IMSIdentityAttribute');
							--					
							SELECT COALESCE(attrValue,@var_tmpstr_notfound) INTO @var_tmpstr
								FROM IMSIdentityAttribute WHERE imsID = @var_imsid AND attrName = 'EnterpriseUsername';
							SET @var_tmpstr2 = @var_new_user_entid;
							IF (@var_tmpstr2 = @var_tmpstr) THEN
								SET @var_tmpstr2 = @var_tmpstr_nochanges;
							ELSE
								SET @var_tmpstr2 = '-> '''||@var_tmpstr2||'''';
							END IF;							
							SET @var_msg_id = @var_msg_id + 1;
							SET @val_message = '  	* EnterpriseUsername : ''' || @var_tmpstr || ''' ' || @var_tmpstr2;
							INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id,@val_message);
							--	
							SELECT COALESCE(attrValue,@var_tmpstr_notfound) INTO @var_tmpstr
								FROM IMSIdentityAttribute WHERE imsID = @var_imsid AND attrName = 'userDomain';
							SET @var_tmpstr2 = @var_new_domain;
							IF (@var_tmpstr2 = @var_tmpstr) THEN
								SET @var_tmpstr2 = @var_tmpstr_nochanges;
							ELSE
								SET @var_tmpstr2 = '-> '''||@var_tmpstr2||'''';
							END IF;							
							SET @var_msg_id = @var_msg_id + 1;
							SET @val_message = '  	* userDomain : ''' || @var_tmpstr || ''' ' || @var_tmpstr2;
							INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id,@val_message);
							--
							SELECT COALESCE(attrValue,@var_tmpstr_notfound) INTO @var_tmpstr
								FROM IMSIdentityAttribute WHERE imsID = @var_imsid AND attrName = 'userEnterpriseDir';
							IF @var_new_ims_entdir_id IS NULL THEN SET @var_new_ims_entdir_id = @var_tmpstr; END IF;							
							SET @var_tmpstr2 = @var_new_ims_entdir_id;
							IF (@var_tmpstr2 = @var_tmpstr) THEN
								SET @var_tmpstr2 = @var_tmpstr_nochanges;
							ELSE
								SET @var_tmpstr2 = '-> '''||@var_tmpstr2||'''';
							END IF;							
							SET @var_msg_id = @var_msg_id + 1;
							SET @val_message = '  	* userEnterpriseDir : ''' || @var_tmpstr || ''' ' || @var_tmpstr2;
							INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id,@val_message);
							--
							SET @var_msg_id = @var_msg_id + 1;
							INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id,'  - IMSIdentityUniqueAttribute');
													
							SELECT COALESCE(attrValue,@var_tmpstr_notfound) INTO @var_tmpstr
								FROM IMSIdentityUniqueAttribute WHERE imsID = @var_imsid AND attrName = 'Enterprise Login';
							SET @var_tmpstr2 = @var_new_ims_user_ent_login;
							IF (@var_tmpstr2 = @var_tmpstr)THEN
								SET @var_tmpstr2 = @var_tmpstr_nochanges;
							ELSE
								SET @var_tmpstr2 = '-> '''||@var_tmpstr2||'''';
							END IF;							
							SET @var_msg_id = @var_msg_id + 1;
							SET @val_message = '  	* Enterprise Login : ''' || @var_tmpstr || ''' ' || @var_tmpstr2;
							INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id,@val_message);
							--
							SELECT COALESCE(attrValue,@var_tmpstr_notfound) INTO @var_tmpstr
								FROM IMSIdentityUniqueAttribute WHERE imsID = @var_imsid AND attrName = 'EnterpriseUpn';
							
							SET @var_tmpstr2 = @var_new_upn;
							IF (@var_tmpstr2 = @var_tmpstr OR @var_tmpstr2 IS NULL)THEN
								SET @var_tmpstr2 = @var_tmpstr_nochanges;
							ELSE
								SET @var_tmpstr2 = '-> '''||@var_tmpstr2||'''';
							END IF;							
							SET @var_msg_id = @var_msg_id + 1;
							SET @val_message = '  	* EnterpriseUpn : ''' || @var_tmpstr || ''' ' || @var_tmpstr2;
							INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id,@val_message);
							--
							SELECT COALESCE(attrValue,@var_tmpstr_notfound) INTO @var_tmpstr
								FROM IMSIdentityUniqueAttribute WHERE imsID = @var_imsid AND attrName = 'userGUID';
							SET @var_tmpstr2 = @var_new_ad_userguid;
							IF (@var_tmpstr2 = @var_tmpstr OR @var_tmpstr2 IS NULL)THEN
								SET @var_tmpstr2 = @var_tmpstr_nochanges;
							ELSE
								SET @var_tmpstr2 = '-> '''||@var_tmpstr2||'''';
							END IF;							
							SET @var_msg_id = @var_msg_id + 1;
							SET @val_message = '  	* userGUID : ''' || @var_tmpstr || ''' ' || @var_tmpstr2;
							INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id,@val_message);
							--	
							SET @var_msg_id = @var_msg_id + 1;
							INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id,'  - IMSTemplatedSyncRepo (affected AA Sync data)');
	
							SELECT CHAR(COUNT(*)) INTO @var_tmpstr FROM IMSTemplatedSyncRepo 
								WHERE attrName = 'account_data' AND entityTypeID = 'scp_user' AND entityID = @var_imsid 
									AND keyValue LIKE '%<auth_id>'||@var_cur_ims_authsrv_id||'</auth_id><aditi_ciuser>'||@var_cur_user_entid||'</aditi_ciuser>%';
							SET @var_msg_id = @var_msg_id + 1;
							SET @val_message = '  	* account_data : ' || @var_tmpstr;
							INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id,@val_message);
							
							SELECT CHAR(COUNT(*)) INTO @var_tmpstr FROM IMSTemplatedSyncRepo 
								WHERE attrName = 'account_data_info' AND entityTypeID = 'scp_user' AND entityID = @var_imsid 
									AND keyValue LIKE '%<auth_id>'||@var_cur_ims_authsrv_id||'</auth_id><aditi_ciuser>'||@var_cur_user_entid||'</aditi_ciuser>%';
							SET @var_msg_id = @var_msg_id + 1;
							SET @val_message = '  	* account_data_info : ' || @var_tmpstr;
							INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id,@val_message);
								
							SELECT CHAR(COUNT(*)) INTO @var_tmpstr FROM IMSTemplatedSyncRepo 
								WHERE attrName = 'policy_object' AND entityTypeID = 'scp_user' AND entityID = @var_imsid 
									AND keyValue LIKE '%<eiid_auth>'||@var_cur_ims_authsrv_id||'</eiid_auth>%';
							SET @var_msg_id = @var_msg_id + 1;
							SET @val_message = '  	* policy_object : ' || @var_tmpstr;
							INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id,@val_message);
							--
							SELECT CHAR(COUNT(*)) INTO @var_tmpstr FROM IMSSoci WHERE imsID = @var_imsid AND sociID LIKE '200%';
							SET @var_msg_id = @var_msg_id + 1;
							SET @val_message = '  - Total Cached Wallet(s) : ' || @var_tmpstr;
							INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id,@val_message);
			
							SELECT CASE WHEN COUNT(*)>0 THEN 'Yes' ELSE 'No' END USE_SMARTCARD INTO @var_tmpstr FROM IMSSoci WHERE imsID = @var_imsid AND sociID LIKE '00D%';
							SET @var_msg_id = @var_msg_id + 1;
							SET @val_message = '  - Using Smart Card : ' || @var_tmpstr;
							INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id,@val_message);
							
						END IF;
					END IF;
				END	IF;

			ELSE
				-- ;
				INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id,'[ERROR] Invalid mode '''|| @IN_operationCmd || '''.');
				SET @var_showSyntax = 'Y';
				--SELECT INTEGER('A') INTO @var_iTmpNum FROM SYSIBM.SYSDUMMY1;
			END IF;
	ELSE
		INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id,'[ERROR] Invalid ISAMESSO schema ''' || CURRENT SCHEMA || '''');
		SET @var_showSyntax = 'Y';		
		--SELECT INTEGER('A') INTO @var_iTmpNum FROM SYSIBM.SYSDUMMY1;
	END IF;
	
	--OPEN @cursor_dbms_output;

END P2;
END P1
#
