-------------------------------------------------------------------
-- Date			: 27-Jul-2012
-- Database		: DB2 9.7
-- Description	: ISAMESSO user domain migration script
-- IMS Version	: 8.1.x
--
-- Dependencies :
--	 - ISAMESSO Enterprise ID / Domain Update Script (sp_base_updEntIdDomainUpnGuid.sql)
--
-- Assumption :-
--   - User account in enterprise directory has been renamed
--
-- Notes :-
--   - Changes on enterprise ID will requires manual changes on the new access profile that rely on enterprise directory
--   - Changes on enterprise id / user domain will trigger password change flow at AA's side
--	 - Change of UPN will affects :
--		 -- Smart Card (00D)
--		 -- AA/AAWWP login with UPN
--
-- Tables affected : 
--  - IMSIdentityAttribute 
--  - IMSIdentityUniqueAttribute
--	- IMSTemplatedSyncRepo
--	- IMSSoci 
--	- IMSSociAttribute
--	- IMSSociUniqueAttribute
--  - IMSCertificate 
--
-- call sp_changeUserEntId ('SIMULATE', 'hlong-test02', 'hlong-test02b', 'qa.encentuate.com', 'hlong-test02@qa.encentuate.com')
-- call sp_changeUserEntId ('SIMULATE', 'hlong-test02', 'hlong-test02b', 'qa.encentuate.com', NULL)
-- call sp_changeUserEntId ('ABOUT', '', '', '', NULL)
-- call sp_changeUserEntId ('HELP', '', '', '', NULL)
--
-- drop procedure sp_changeUserEntId
--
CREATE PROCEDURE sp_changeUserEntId
(
 @IN_operationCmd VARCHAR(8)
 
,@IN_curUserEntId VARCHAR(128)
,@IN_newUserEntId VARCHAR(128)

,@IN_curDomain VARCHAR(128)
,@IN_newUPN VARCHAR(128)
)
DYNAMIC RESULT SETS 1
MODIFIES SQL DATA
LANGUAGE SQL 

P1 : BEGIN
		DECLARE @var_exec_id VARCHAR (128);
		DECLARE @var_msg_id INTEGER DEFAULT 1;
		DECLARE TMPTBL_ALREADY_EXIST CONDITION FOR SQLSTATE '42710';
		DECLARE CONTINUE HANDLER FOR TMPTBL_ALREADY_EXIST
		BEGIN
		END;
		DECLARE GLOBAL TEMPORARY TABLE SESSION.tmpDbmsOutput
		(
			EXECID VARCHAR (32) NOT NULL,
			MSGID INTEGER NOT NULL DEFAULT 0,
			MSG VARCHAR (1024) NOT NULL
		) NOT LOGGED ON COMMIT PRESERVE ROWS;	
				
		SELECT CHAR(CURRENT TIMESTAMP) INTO @var_exec_id FROM SYSIBM.SYSDUMMY1;

P2 : BEGIN
	
	DECLARE @var_showSyntax CHAR(1) DEFAULT 'N';
	DECLARE @var_scriptName VARCHAR(64) DEFAULT 'sp_changeUserEntId';
	DECLARE @var_scriptDesc VARCHAR(64) DEFAULT 'Rename Enterprise ID';
	DECLARE @var_scriptVer VARCHAR(32) DEFAULT '1.0.0';
	DECLARE @var_tmpStr VARCHAR(256);
	DECLARE @exception_msg VARCHAR(128) DEFAULT NULL;

	DECLARE BASESP_NOT_FOUND CONDITION FOR SQLSTATE '42884';
	
	DECLARE @var_new_upn VARCHAR(256);
	DECLARE @var_new_ad_userguid VARCHAR(256);
	
	DECLARE @cursor_dbms_output CURSOR WITH RETURN TO CLIENT FOR
	SELECT MSG Execution_Output FROM SESSION.tmpDbmsOutput WHERE EXECID = @var_exec_id ORDER BY MSGID;
	
	DECLARE EXIT HANDLER FOR SQLEXCEPTION
	BEGIN
		GET DIAGNOSTICS EXCEPTION 1 @exception_msg = MESSAGE_TEXT;
		SET @var_msg_id = @var_msg_id + 1;
		SET @var_tmpStr = '   [ERROR] Missing dependency. Please make sure ISAMESSO stored procedure ''sp_base_updEntIdDomainUpnGuid'' is created in the same schema.';
		INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id,@exception_msg);		
		OPEN @cursor_dbms_output;
	END;
	
	IF(TRIM(@IN_newUPN) = '')THEN
		SET @var_new_upn = NULL;
	ELSE 
		SET @var_new_upn = @IN_newUPN;
	END IF;
	
	
	SET @var_tmpStr = 'command : '||@var_scriptName||'('''||@IN_operationCmd||''', '''||@IN_curUserEntId||''', '''||@IN_newUserEntId||''', '''||@IN_curDomain||''', '||COALESCE(''''||@IN_newUPN||'''','NULL')||')';
	INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id, @var_tmpStr);

	call sp_base_updEntIdDomainUpnGuid(@IN_operationCmd, @IN_curUserEntId, @IN_curDomain, @IN_newUserEntId, @IN_curDomain , @var_new_upn, NULL
		,@var_showSyntax 
		,@var_exec_id,@var_scriptName, @var_scriptDesc, @var_scriptVer);
	
	IF @var_showSyntax = 'Y' THEN
		SET @var_msg_id = @var_msg_id + 1000;
		INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id ,'');
		INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id+1 ,' syntax : '||@var_scriptName||'(''<mode>'', ''<user id>'', ''<current user domain>'', ''<new user domain>'', ''<new AD userGUID>'', ''<new upn>'')');
		INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id+2 ,' * <mode> - Mandatory, text, case sensitive'); 
		INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id+3 ,'          - ''ABOUT'' display script information');
		INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id+4 ,'          - ''HELP'' display script syntax and example');
		INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id+5 ,'          - ''UPDATE'' to perform actual rename');
		INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id+6 ,'          - ''SIMULATE'' to check on the target enterprise user id without performing actual migration'); 
		INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id+7 ,' * <current user id> - Mandatory, text, case sensitive');
		INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id+8 ,'                     - Target user enterprise ID');
		INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id+9 ,' * <new user id> - Mandatory, text, case sensitive');
		INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id+10 ,'                 - Target user enterprise ID');
		INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id+11,' * <user domain> - Mandatory, text, case sensitive');
		INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id+12,'                 - current user domain');
		INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id+13,' * <new upn> - Mandatory, text, case sensitive');
		INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id+14,'             - Enter ''NULL'' if not applicable or value remain unchanged');
		INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id+15,'');
		INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id+16,'Example : ');
		INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id+17,' * sp_changeUserEntId (''SIMULATE'',''ims001'', ''ims002'',''current.domain.com'', ''ims002@current.domain.com'')');
		INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id+18,'   - Simulate user ''ims001'' enterprise ID & UPN changes without performing actual operation');
		INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id+19,'');
		INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id+20,' * sp_changeUserEntId (''SIMULATE'',''ims001'', ''ims002'',''current.domain.com'', NULL)');
		INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id+21,'   - Simulate user ''ims001'' enterprise ID changes without performing actual operation');
		INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id+22,'');
		INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id+23,' * sp_changeUserEntId (''UPDATE'',''ims001'', ''ims002'',''current.domain.com'', ''ims002@current.domain.com'')');
		INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id+24,'   - Rename user ''ims001'' from ''current.domain.com'' to ''ims002'' and update UPN to ''ims002@current.domain.com''');
		INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id+25,'');
		INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id+26,' * sp_changeUserEntId (''UPDATE'',''ims001'', ''ims002'',''current.domain.com'', NULL)');
		INSERT INTO SESSION.tmpDbmsOutput(EXECID,MSGID,MSG) VALUES (@var_exec_id,@var_msg_id+27,'   - Rename user ''ims001'' from ''current.domain.com'' to ''ims002'' without update UPN');		
	END IF;
	
	OPEN @cursor_dbms_output;
	
END P2; 
END P1
#

